@@ -1,11 +1,14 @@
-#ifndef GNUC_ATTRIBUTE_CHECK /* disable GNUC attribute checking if not GNUC */
-#ifdef  __attribute__        /* Avoid possible redefinition errors */
+#ifndef NEXT30_NO_ATTRIBUTE
+#ifndef HASATTRIBUTE       /* disable GNU-cc attribute checking? */
+#ifdef  __attribute__      /* Avoid possible redefinition errors */
 #undef  __attribute__
-#endif  
+#endif
 #define __attribute__(attr)
+#endif 
 #endif
 #ifdef OVERLOAD
 SV*	amagic_call _((SV* left,SV* right,int method,int dir));
+bool Gv_AMupdate _((HV* stash));
 #endif /* OVERLOAD */
 OP*	append_elem _((I32 optype, OP* head, OP* tail));
 OP*	append_list _((I32 optype, LISTOP* first, LISTOP* last));
@@ -40,6 +43,7 @@ OP *	ck_retarget _((OP *op));
 OP*	convert _((I32 optype, I32 flags, OP* op));
 char*	cpytill _((char* to, char* from, char* fromend, int delim, I32* retlen));
 void	croak _((char* pat,...)) __attribute__((format(printf,1,2),noreturn));
+CV*	cv_clone _((CV* proto));
 void	cv_undef _((CV* cv));
 #ifdef DEBUGGING
 void	cx_dump _((CONTEXT* cs));
@@ -91,7 +95,7 @@ void	do_vecset _((SV* sv));
 void	do_vop _((I32 optype, SV* sv, SV* left, SV* right));
 void	dump_all _((void));
 void	dump_eval _((void));
-#ifdef NOTDEF  /* See util.c */
+#ifdef DUMP_FDS  /* See util.c */
 int	dump_fds _((char* s));
 #endif
 void	dump_form _((GV* gv));
@@ -125,7 +129,7 @@ void	he_delayfree _((HE* hent));
 void	he_free _((HE* hent));
 void	hoistmust _((PMOP* pm));
 void	hv_clear _((HV* tb));
-SV*	hv_delete _((HV* tb, char* key, U32 klen));
+SV*	hv_delete _((HV* tb, char* key, U32 klen, I32 flags));
 bool	hv_exists _((HV* tb, char* key, U32 klen));
 SV**	hv_fetch _((HV* tb, char* key, U32 klen, I32 lval));
 I32	hv_iterinit _((HV* tb));
@@ -206,7 +210,10 @@ char*	my_bcopy _((char* from, char* to, I32 len));
 #if !defined(HAS_BZERO) && !defined(HAS_MEMSET)
 char*	my_bzero _((char* loc, I32 len));
 #endif
-void	my_exit _((I32 status)) __attribute__((noreturn));
+void	my_exit _((U32 status)) __attribute__((noreturn));
+#ifdef	USE_MY_FMOD
+double	my_fmod _((double x, double y));
+#endif
 I32	my_lstat _((void));
 #ifndef HAS_MEMCMP
 I32	my_memcmp _((unsigned char* s1, unsigned char* s2, I32 len));
@@ -217,14 +224,15 @@ void	my_setenv _((char* nam, char* val));
 I32	my_stat _((void));
 #ifdef MYSWAP
 short	my_swap _((short s));
+long	my_htonl _((long l));
+long	my_ntohl _((long l));
 #endif
 void	my_unexec _((void));
 OP*	newANONLIST _((OP* op));
 OP*	newANONHASH _((OP* op));
 OP*	newANONSUB _((I32 floor, OP* block));
 OP*	newASSIGNOP _((I32 flags, OP* left, I32 optype, OP* right));
-OP*	newBINOP _((I32 optype, I32 flags, OP* left, OP* right));
-OP*	newCONDOP _((I32 flags, OP* expr, OP* true, OP* false));
+OP*	newCONDOP _((I32 flags, OP* expr, OP* trueop, OP* falseop));
 void	newFORM _((I32 floor, OP* op, OP* block));
 OP*	newFOROP _((I32 flags, char* label, line_t forline, OP* scalar, OP* expr, OP*block, OP*cont));
 OP*	newLOGOP _((I32 optype, I32 flags, OP* left, OP* right));
@@ -238,7 +246,6 @@ OP*	newRANGE _((I32 flags, OP* left, OP* right));
 OP*	newSLICEOP _((I32 flags, OP* subscript, OP* list));
 OP*	newSTATEOP _((I32 flags, char* label, OP* o));
 CV*	newSUB _((I32 floor, OP* op, OP* block));
-OP*	newUNOP _((I32 optype, I32 flags, OP* child));
 CV*	newXS _((char *name, void (*subaddr)(CV* cv), char *filename));
 #ifdef DEPRECATED
 CV*	newXSUB _((char *name, I32 ix, I32 (*subaddr)(int,int,int), char *filename));
@@ -299,7 +306,7 @@ I32	perl_callpv _((char* subname, I32 sp, I32 gimme, I32 hasargs, I32 numargs));
 I32	perl_callsv _((SV* sv, I32 sp, I32 gimme, I32 hasargs, I32 numargs));
 #endif
 void	perl_construct _((PerlInterpreter* sv_interp));
-void	perl_destruct _((PerlInterpreter* sv_interp, int destruct_level));
+void	perl_destruct _((PerlInterpreter* sv_interp));
 void	perl_free _((PerlInterpreter* sv_interp));
 SV*	perl_get_sv _((char* name, I32 create));
 AV*	perl_get_av _((char* name, I32 create));
@@ -378,15 +385,12 @@ OP*	scalarvoid _((OP* op));
 unsigned long	scan_hex _((char* start, I32 len, I32* retlen));
 char*	scan_num _((char* s));
 unsigned long	scan_oct _((char* start, I32 len, I32* retlen));
-#ifdef NOTDEF /* See toke.c pp_ctl.c and op.c */
-void	scan_prefix _((PMOP* pm, char* string, I32 len));
-#endif
 OP*	scope _((OP* o));
 char*	screaminstr _((SV* bigsv, SV* littlesv));
 #ifndef VMS
 I32	setenv_getix _((char* nam));
 #endif
-VOIDRET sighandler _((int sig));
+Signal_t sighandler _((int sig));
 SV**	stack_grow _((SV** sp, SV**p, int n));
 int	start_subparse _((void));
 bool	sv_2bool _((SV* sv));
